@@ -5,9 +5,9 @@ use warnings;
 use List::Util qw/any/;
 use Scalar::Util qw/looks_like_number/;
 
-our $VERSION='0.1.5';
+our $VERSION='0.1.6';
 
-my %property=map {$_=>undef} qw/tmmin tmavg tmmax next finish message attribute note attributes/;
+my %property=map {$_=>undef} qw/tmmin tmavg tmmax next finish message attribute note attributes require/;
 
 my %defaults=(
 	'tmmax/tmavg'=>5/4,
@@ -95,12 +95,17 @@ sub increment {
 sub nextrandom {
 	my ($self,%opt)=@_;
 	if(!$$self{next}) { return }
-	my $N=1+$#{$$self{next}};
-	if($N<=0) { return }
-	if($N==1) { return $$self{next}[0] }
-	my $node=$$self{next}[ int(rand($N)) ];
-	if($opt{not}) { while($node eq $opt{not}) { $node=$$self{next}[ int(rand($N)) ] } }
-	return $node;
+	if($#{$$self{next}}==0) { return $$self{next}[0] } # this shouldn't be needed
+	my @candidates;
+	foreach my $next (@{$$self{next}}) {
+		if($opt{not}&&($opt{not} eq $next)) { next }
+		if(!ref($next)) { push @candidates,$next; next }
+		if($$next{require}&&$opt{attr}) {
+			if(!$$next{require}->matches($opt{tm},%{$opt{attr}})) { next } }
+		push @candidates,$next;
+	}
+	if(!@candidates) { return }
+	return $candidates[ int(rand(1+$#candidates)) ];
 }
 
 sub hasnext {
